* -------------------------- BEGIN HistFreq -------------------------
      subroutine HistFreq(y,n,
     :       spec_bin, bin_start, bin_width, bin_stop,
     :       bin_low, bin_center, bin_high, nbins,
     :       ymin, ymax,
     :       bin_count, bin_percent, 
     :       cumulative_count, cumulative_percent)

! Returns numbers in each bin for values in array y (which need not be sorted).  
! There are several ways of specifying the bins, as shown in this table:

! spec_bin  bin_start             bin_width          bin_stop   comment
!        T      input                 input input > bin_start   use specified inputs
!        T      input                 input  input==bin_start   compute bin_stop, nbins
!        F calc(miny) calc(from nbins input)       calc(maxy)   find bin_start, bin_stop, bin_width

! Note that y values less than bin_start or greater than bin_stop will not be counted.

! In CoPlot only the value for the lower bound of a bin is given, assuming all
! bins are contiguous.  At the expense of an additional, usually redundant, variable
! I will use variables for the lower and higher bounds of each bin (thus
! I could have noncontiguous bins if desired in the future).

! The bin centers are returned in bin_center and the number in icount.

! Note that icount is the number of data values such that:
!    bin_left <= count(data) < bin_h 
! (bin_l = the lower y-value of the bin; bin_h = the upper y-value of the bin)
! (this is the convention used by CoStat).

! "Cumulative_percent" is calculated to be the percent of the data below bin_h
! so that 100 - cumulative is the percent of the data in and above bin_h

! Must specify dimensions of the output arrays large enough to allow for the
! number of bins (which might not be known when the routine is called).

* Note: if want to compare the counts from this subroutine with manual
* counts from a file of the data, must have correspondence between the number
* of decimal places used in the printout of the data and that used in this
* subroutine (this is critical for values that appear on the printout to be at
* bin boundaries).  One simple way to guarantee this is to write and read 
* the data to a character buffer using the desired printout format.  This will 
* round the data value in a consistent way.  Here is an example:
* Convert y_in to 2 decimal places (store as y2dec) within the calling program
! in order to better check the counts obtained using histfreq:
*        character buf5*5
*        buf5 = ' '
*        write(buf5,'(f5.2)') y_in
*        read(buf5,'(f5.2)') y2dec

* Dates: 09/04/98 - Written by D. Boore
*        09/08/98 - Use indexx rather than hpsort, in order not to rearrange
*                   the y array.
*        07/05/01 - Added cumulative sum to output and bin_stop to input
*        07/06/01 - Revised cumulative to be the percent of the data below
*                   bin_l
*        06/16/02 - Minor modification of the explanatory text above.
*                   Use allocatable array for indx
*        04/18/03 - Found exceeded array allocation in do whiles, so added
*                   some checks for the index exceeding the allocated size.
*                   The program is somewhat klutzy; I should redo the 
*                   code sometime.
!        05/18/10 - Cleanup code.
!        06/03/10 - Numerous changes

      real y(*), bin_low(*), bin_center(*), bin_high(*)
      integer bin_count(*), cumulative_count(*) 
      real bin_percent(*), cumulative_percent(*) 
      
      integer indx(:)
      allocatable :: indx

      logical spec_bin

      allocate( indx(n))

* Sort array:

      call indexx(n,y,indx)
      
      ymin = y(indx(1))
      ymax = y(indx(n))
      call mnmax(y,1,n,1,ymin_mnmax,ymax_mnmax) 
      
!DEBUG
      write(*,*) ' ymin, ymin_mnmax, ymax, ymax_mnmax = ',
     :             ymin, ymin_mnmax, ymax, ymax_mnmax
!DEBUG
      
* Compute bin info:

! spec_bin  bin_start             bin_width          bin_stop   nbins     comment
!        T      input                 input   input>bin_start   0.0    redefine bin_stop, calc nbins
!        T      input                   0.0   input>bin_start   input  compute bin_width
!        T      input                 input  input==bin_start   input  compute bin_stop
!        T      input                 input  input==bin_start   0.0    compute bin_stop (from ymax), nbins
!        F calc(miny) calc(from nbins input)       calc(maxy)   input  find bin_start, bin_stop, bin_width

!DEBUG
      write(*,*) ' On input, bin_start, bin_width, bin_stop, nbins = ',
     :             bin_start, bin_width, bin_stop, nbins
!DEBUG
      
      if (.not. spec_bin) then
        bin_start = ymin   ! start at the minimum value
        bin_stop  = ymax   ! start at the minimum value
        bin_width = (bin_stop - bin_start)/real(nbins)
      else
        if (bin_stop > bin_start .and. bin_width /= 0.0) then
          nbins = int((bin_stop - bin_start)/bin_width)+1
          bin_stop = bin_start + real(nbins)*bin_width    ! redefine bin_stop:
        else if (bin_stop > bin_start .and. bin_width == 0.0) then
          bin_width = (bin_stop - bin_start)/real(nbins)
        else if (bin_stop == bin_start .and. nbins /= 0) then
          bin_stop = bin_start + real(nbins)*bin_width
        else
          nbins = int((ymax - bin_start)/bin_width)+1
          bin_stop = bin_start + real(nbins)*bin_width    ! redefine bin_stop:
        end if
      end if
      
!DEBUG
      write(*,*) 
     :' After if statements, bin_start, bin_width, bin_stop, nbins = ',
     :                        bin_start, bin_width, bin_stop, nbins
!DEBUG
      
! fill bin arrays:

      
      do i = 1, nbins
        bin_low(i) = 0.0
        bin_center(i) = 0.0
        bin_high(i) = 0.0
        bin_count(i) = 0
        bin_percent(i) = 0.0 
        cumulative_count(i) = 0
        cumulative_percent(i) = 0.0
      end do
      
      do i = 1, nbins
        bin_low(i)  = bin_start + real(i-1)*bin_width
        bin_high(i) = bin_low(i) + bin_width
        bin_center(i) = 0.5*(bin_low(i) + bin_high(i))
      end do

      loop over bins: DO i = 1, nbins

! Find counts:

! This is the brute force way of finding the count per bin, making no
! assumptions about the order of the y values

        icount = 0
        DO j = 1, n
          IF (y(j) >= bin_low(i) .and. y(j) < bin_high(i) ) THEN
            icount = icount + 1
          END IF
        END DO
            
        bin_count(i) = icount
        
        IF (i == 1) THEN
          cumulative_count(i) = bin_count(i)
        ELSE
          cumulative_count(i) = cumulative_count(i-1) + bin_count(i)
        END IF
        
      END DO loop over bins
        
! normalize:

      DO i = 1, nbins
        bin_percent(i) = 100.0*bin_count(i)/cumulative_count(nbins)
        cumulative_percent(i) = 
     :            100.0*cumulative_count(i)/cumulative_count(nbins)
      END DO
        
 
      deallocate ( indx)
    
      return
      end
* -------------------------- END HistFreq -------------------------

!      include '\forprogs\indexx.for'
!      include '\forprogs\mnmax.for'
